#ifndef covering_h
#define covering_h

#include "localMap.h"

//////////////////////////////
// Function: isCovering()
// 
// Purpose: Validates that N => M in the setting where N[0] and M[0] is the exit coordinate and N[1], M[1], N[2], M[2]
// are entry coordinates.
bool isCovering(const localMap &f,IVector N,IVector &M);

//////////////////////////////
// Function: isCovering()
// 
// Purpose: Validates that N => M in the setting where N[0],N[1],M[0] and M[1] are the exit coordinates and N[2], M[2]
// are entry coordinates.
bool isCovering2(const localMap &f,IVector N,IVector &M);

/////////////////////////////
// Function: propagateCone()
//
// Purpose: The h-set N is equipped with a cone "Cone". The cone is represented as
//   Cone = 1 x Cone[1] x Cone[2]
// wher Cone[1] and Cone[2] are intervals centered at zero. The function returns another
// cone in set M, using the same representation. The cone is computed using Lemma 7 from the paper.
//
// Note: As the function is executed the set M itself is also computed. In more detail:
//
// The size of the set M along the exit coordinate (the first coordinate; of index 0) is fixed.
// Along this coordinate we validate that the images of the left and the right components of the
// exit set of N are mapped to the two opposite sides of M[0].
//
// The second coordinate (of index 1) is the entry coordinate associated with the weak expansion.
// Along this coordinate the set M needs to be enlarged, so that (f(N))[1] is a subset of M[1].
// 
// The last coordinate of the set M, M[2] is fixed. We chack that (f(N))[2] is a subset of M[2]
// to validate the entry condition.
IVector propagateCone(IVector N,IVector Cone,const localMap &f);

/////////////////////////////
// Function: propagateCone()
// 
// Purpose: This function considers a sequence of h-sets and a sequence of local maps. 
// The sequence of h-sets is associated with a covering
//   N0 => ... => Nk
// The function takes a cone in N0 and computes a cone in Nk, such that for each
// local map fi:Ni->Ni+1 the cone condition is satisfied.
IVector propagateCone(vector<IVector> N,IVector Cone,const vector<localMap> &f);

/////////////////////////////
// Function: checkHyperbolicity()
//
// Purpose: This function validates the condition that every matrix from (21) from the paper
// is strictly positive definite. This is for a single local map f:N->R^3
bool checkHyperbolicity(const localMap &f,IVector N);

/////////////////////////////
// Function: checkHyperbolicity()
//
// Purpose: This function validates the condition that every matrix from (21) from the paper
// is strictly positive definite for every local map associated with a sequence of coverings
//   N0 => ... => Nk
bool checkHyperbolicity(const vector<localMap> &f,vector<IVector> N);

#endif