#ifndef localMap_h
#define localMap_h

#include "capd/capdlib.h"
#include <iostream>
using namespace std;
using namespace capd;
using namespace capd::matrixAlgorithms;

/////////////////////////////
// class localMap
//
// Purpose: To compute bounds on the image and on the derivative of a map f expressed in affine local coordinates.
//
// A local change of coordinate is affine, and given by a vector and matrix. 
// We have two local coordinates given by vectors and matrices
//  q0, A0 - for the first local coordinate
//  q1, A1 - for the second local coordinate
// The local map is 
//   LocalMap(x) = A1inv*(f(A0*x+q0)-q1)
// and its derivative is
//   DLocalMap(x) = A1inv*Df(A0*x+q0)*A0
// 
// The class is equipped with two operators:
// The operator () is used to compute images of sets.
// The operator [] is used to compute the derivative of the map.
//
// The function of main interest is image(), which computes both the derivative of the map and the image.
// The derivative is passed by reference to D and image() returns the image.
class localMap
{
private:
	IMap *f;
	IVector q0,q1;
	IMatrix A0,A1,A1inv;
	
public:
	IVector image(const IVector &x,IMatrix &D) const;
	
	IVector operator()(const IVector &x,IMatrix &D) const {return image(x,D);}
	IVector operator()(const IVector &x) const {IMatrix dummy; return image(x,dummy);}
	IMatrix operator[](const IVector &x) const {IMatrix D; image(x,D); return D;}
	
	localMap(IMap &f,IVector q0,IVector q1,IMatrix A0,IMatrix A1);
	localMap(){}
};

#endif