#include "localMapsFromData.h"

int CoveringSequenceLength(int i)
{
	if(i==1) return 5;
	if(i==2) return 4;
	return 0;
}

//////////////////////////////////

// This is a technical function which downloads the X,Y
// coordinates of the points for the local coordinates.
// It is later used to choose the points along the Z coordinate
// based on the procedu re described in section 4.1.
vector<DVector> Dpoints(int i)
{
	ifstream file("data/points"+to_string(i)+".txt");
	int N=CoveringSequenceLength(i);

	vector<DVector> q(N+1);
	
	double a;
	for(int n=0;n<N;n++)
	{
		q[n]=DVector(3);
		file >> a;
		q[n][0]=a;
		file >> a;
		q[n][1]=a;
	}
	file.close();
	q[N]=q[0];
	return q;
}

// This function is used to compute l1 and l2 from (9) and (10)
// from the paper. In fact 
//   l1(z,xi)=G(z,xi,q1), where q1 is a sequence of points along the first sequence
//   l2(z,xi)=G(z,xi,q2), where q2 is a sequence of points along the second sequence
double G(double z,double xi,const vector<DVector> &q)
{
	int k=q.size()-1;
	for(int i=0;i<k;i++) z=xi*z+q[i][1];
	return z;
}

// This is the derivative of G with respect to z.
double DG(double z,double xi,const vector<DVector> &q)
{
	double d=1.0;
	int k=q.size()-1;
	for(int i=0;i<k;i++) d=xi*d;
	return d;
}

// This G returns (l1(z,xi)-l2(z,xi))/2-z
double G(double z,double xi,const vector<DVector> &q0,const vector<DVector> &q1)
{
	return z-0.5*(G(z,xi,q0)+G(z,xi,q1));
}
double DG(double z,double xi,const vector<DVector> &q0,const vector<DVector> &q1)
{
	return 1.0-0.5*(DG(z,xi,q0)+DG(z,xi,q1));
}

double pM(double xi)
{
	double z=-20;
	vector<DVector> q1=Dpoints(1);
	vector<DVector> q2=Dpoints(2);
	
	// We use the Newton method to obtain a numerical solution of
	// (l1-l2)/2-z = 0
	for(int i=0;i<10;i++) z=z-G(z,xi,q2,q1)/DG(z,xi,q2,q1);
	return z;
}

//////////////////////////////

vector<IVector> points(int i,interval xi)
{
	ifstream file("data/points"+to_string(i)+".txt");
	int N=CoveringSequenceLength(i);

	vector<IVector> q(N+1);
	
	double a;
	for(int n=0;n<N;n++)
	{
		q[n]=IVector(3);
		file >> a;
		q[n][0]=a;
		file >> a;
		q[n][1]=a;
	}
	file.close();
	
	q[0][2]=pM((xi.mid()).leftBound());
	for(int k=0;k<N-1;k++) q[k+1][2]=(xi*q[k][2]+q[k][1]).mid();
	
	return q;
}

//////////////////////////////////

vector<IMatrix> coordinates(int i)
{
	ifstream file("data/A"+to_string(i)+".txt");
	int N=CoveringSequenceLength(i);
	
	vector<IMatrix> A(N+1);
	double a;
	for(int n=0;n<N;n++)
	{
		A[n]=IMatrix(3,3);
		for(int i=0;i<3;i++)
			for(int k=0;k<3;k++)
		{
			file >> a;
			A[n][i][k]=a;
		}
	}
	file.close();
	
	return A;
}

vector<localMap> getLocalMaps(interval xi,IMap &f,int i)
{
	f.setParameter("xi",xi);
	
	vector<IMatrix> A=coordinates(i);
	vector<IVector> q=points(i,xi.mid());
	
	// Here we are making surre that we start and end always with the same coordinate system:
	vector<IMatrix> A1=coordinates(1);
	vector<IVector> q1=points(1,xi.mid());
	IMatrix AM=A1[0];
	IVector pM=q1[0];
	int N=q.size()-1;
	A[0]=AM;
	A[N]=AM;
	q[0]=pM;
	q[N]=pM;
	
	vector<localMap> F(N);
	for(int n=0;n<N;n++) F[n]=localMap(f,q[n],q[n+1],A[n],A[n+1]);
	
	return F;
}